library poller;

class Poller {
  final Function callback;
  final Function callbackAsync;
  final int days;
  final int hours;
  final int minutes;
  final int seconds;
  final int milliseconds;
  final int microseconds;
  final bool logging;
  
  bool _paused = true;
  bool _stopped = true;
  Future _timer;

  /// Generates a new Poller object
  /// Takes the following optional parameters:
  ///   * callback: Function
  ///   * callbackAsync: async Function
  ///   * days: int
  ///   * hours: int
  ///   * minutes: int
  ///   * seconds: int
  ///   * milliseconds: int
  ///   * microseconds: int
  ///   * logging: int
  /// Either callback or callbackAsync must not be null.
  /// The Poller will execute after the sum total of all the days, hours,
  /// minutes, seconds, milliseconds, and microseconds parameters time has passed.
  /// Logging will enable a few print statements to enable superior visibility
  /// into the actions of the Poller.
  Poller({
    this.callback,
    this.callbackAsync,
    this.days = 0,
    this.hours = 0,
    this.minutes = 0,
    this.seconds = 0,
    this.milliseconds = 0,
    this.microseconds = 0,
    this.logging = false
  }) {
    assert(this.days != null);
    assert(this.hours != null);
    assert(this.minutes != null);
    assert(this.seconds != null);
    assert(this.milliseconds != null);
    assert(this.microseconds != null);
    assert(this.logging != null);
    assert(this.callback != null || this.callbackAsync != null);
  }

  /// Starts the Poller object.
  void start() {
    if (this.logging) {
      print('Starting');
    }
    this._paused = false;
    this._stopped = false;
    this._timer = _generateTimer();
  }

  /// Stops the Poller object permanently. Must use Poller.start() to begin again.
  void stop() {
    if (this.logging) {
      print('Stopping');
    }
    this._paused = true;
    this._stopped = true;
    this._timer = null;
  }

  /// Temporarily pauses the Poller object. Must use Poller.resume() to begin again.
  void pause() {
    if (this.logging) {
      print('Pausing');
    }
    this._paused = true;
  }

  /// Resumes the Poller object.
  void resume() {
    if (this.logging) {
      print('Resuming');
    }
    this._paused = false;
  }

  bool get isPaused => _paused;

  bool get isStopped => _stopped;

  Future get timer => _timer;

  Future _generateTimer() {
    if (!_stopped) {
      return Future.delayed(
          Duration(
              days: this.days,
              hours: this.hours,
              minutes: this.minutes,
              seconds: this.seconds,
              milliseconds: this.milliseconds,
              microseconds: this.microseconds
          ), () async {
        if (!this._paused) {
          if (this.logging) {
            print('Executing');
          }
          _execute();
          await _executeAsync();
        }
        else {
          if (this.logging) {
            print('Paused - not executing');
          }
        }
        this._timer = _generateTimer();
      });
    }
    return null;
  }

  void _execute() {
    if (this.callback != null) {
      this.callback();
    }
  }
  
  Future<void> _executeAsync() async {
    if (this.callbackAsync != null) {
      await this.callbackAsync();
    }
  }
}