import 'package:flutter_test/flutter_test.dart';

import 'package:poller/poller.dart';

void main() {
  const int pollerSeconds = 3;
  const int initialValue = 0;
  const bool verbose = true;
  const Duration waitTime = Duration(seconds: pollerSeconds, milliseconds: 200);

  test('executes callback after x seconds', () async {
    if (verbose) {
      print('Starting \'executes callback after x seconds\'');
    }
    int value = initialValue;
    final Poller poller = Poller(seconds: pollerSeconds, callback: () {
      value += 1;
    });
    poller.start();

    expect(value, initialValue);
    if (verbose) {
      print('Maintained initial value');
    }
    await Future.delayed(waitTime, () {
      expect(value, initialValue + 1);
      if (verbose) {
        print('Updated to new value');
        print('');
      }
    });
  });

  test('executes callbackAsync after x seconds', () async {
    if (verbose) {
      print('Starting \'executes callbackAsync after x seconds\'');
    }
    int value = initialValue;
    final Poller poller = Poller(seconds: pollerSeconds, callbackAsync: () async {
      value += 1;
    });
    poller.start();

    expect(value, initialValue);
    if (verbose) {
      print('Maintained initial value');
    }
    await Future.delayed(waitTime, () {
      expect(value, initialValue + 1);
      if (verbose) {
        print('Updated to new value');
        print('');
      }
    });
  });

  test('resets after x seconds', () async {
    if (verbose) {
      print('Starting \'resets after x seconds\'');
    }
    int value = initialValue;
    final Poller poller = Poller(seconds: pollerSeconds, callback: () {
      value += 1;
    });
    poller.start();

    expect(value, initialValue);
    if (verbose) {
      print('Maintained initial value');
    }
    await Future.delayed(waitTime, () {
      expect(value, initialValue + 1);
      if (verbose) {
        print('Updated to new value once');
      }
    });
    await Future.delayed(waitTime, () {
      expect(value, initialValue + 2);
      if (verbose) {
        print('Updated to new value twice');
        print('');
      }
    });
  });

  test('pauses', () async {
    if (verbose) {
      print('Starting \'pauses\'');
    }
    int value = initialValue;
    final Poller poller = Poller(seconds: pollerSeconds, callback: () {
      value += 1;
    });
    poller.start();

    expect(value, initialValue);
    if (verbose) {
      print('Maintained initial value');
    }
    expect(poller.isPaused, false);
    poller.pause();
    expect(poller.isPaused, true);
    if (verbose) {
      print('Set _paused');
    }
    await Future.delayed(waitTime, () {
      expect(value, initialValue);
      if (verbose) {
        print('Maintained initial value once');
      }
    });
    await Future.delayed(waitTime, () {
      expect(value, initialValue);
      if (verbose) {
        print('Maintained initial value twice');
        print('');
      }
    });
  });

  test('resumes', () async {
    if (verbose) {
      print('Starting \'resumes\'');
    }
    int value = initialValue;
    final Poller poller = Poller(seconds: pollerSeconds, callback: () {
      value += 1;
    });
    poller.start();

    expect(value, initialValue);
    if (verbose) {
      print('Maintained initial value');
    }
    expect(poller.isPaused, false);
    poller.pause();
    expect(poller.isPaused, true);
    if (verbose) {
      print('Set _paused');
    }
    await Future.delayed(waitTime, () {
      expect(value, initialValue);
      if (verbose) {
        print('Maintained initial value once');
      }
    });
    await Future.delayed(waitTime, () {
      expect(value, initialValue);
      if (verbose) {
        print('Maintained initial value twice');
      }
    });

    poller.resume();
    expect(poller.isPaused, false);
    await Future.delayed(waitTime, () {
      expect(value, initialValue + 1);
      if (verbose) {
        print('Updated to new value once');
      }
    });
    await Future.delayed(waitTime, () {
      expect(value, initialValue + 2);
      if (verbose) {
        print('Updated to new value twice');
        print('');
      }
    });
  });

  test('stops', () async {
    if (verbose) {
      print('Starting \'stops\'');
    }
    int value = initialValue;
    final Poller poller = Poller(
      seconds: pollerSeconds,
      callback: () {
        value += 1;
      },
      logging: true
    );
    expect(poller.isStopped, true);
    poller.start();
    expect(poller.isStopped, false);

    expect(value, initialValue);
    if (verbose) {
      print('Maintained initial value');
    }
    await Future.delayed(waitTime, () {
      expect(value, initialValue + 1);
      if (verbose) {
        print('Updated to new value once');
      }
    });
    await Future.delayed(waitTime, () {
      expect(value, initialValue + 2);
      if (verbose) {
        print('Updated to new value twice');
      }
    });

    expect(poller.isStopped, false);
    poller.stop();
    expect(poller.isStopped, true);
    expect(poller.timer, null);

    await Future.delayed(waitTime, () {
      expect(value, initialValue + 2);
      if (verbose) {
        print('Maintained updated value once');
      }
    });
    await Future.delayed(waitTime, () {
      expect(value, initialValue + 2);
      if (verbose) {
        print('Maintained updated value twice');
        print('');
      }
    });
  });

}
